# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

from . import _utilities
import typing
# Export this package's modules as members:
from .abs import *
from .abspath import *
from .alltrue import *
from .anytrue import *
from .base64decode import *
from .base64encode import *
from .base64gzip import *
from .base64sha256 import *
from .base64sha512 import *
from .basename import *
from .bcrypt import *
from .ceil import *
from .chomp import *
from .chunklist import *
from .cidrhost import *
from .cidrnetmask import *
from .cidrsubnet import *
from .coalesce import *
from .coalescelist import *
from .compact import *
from .concat import *
from .contains import *
from .csvdecode import *
from .dirname import *
from .distinct import *
from .element import *
from .endswith import *
from .file import *
from .filebase64 import *
from .filebase64sha256 import *
from .filebase64sha512 import *
from .fileexists import *
from .filemd5 import *
from .filesha1 import *
from .filesha256 import *
from .filesha512 import *
from .flatten import *
from .floor import *
from .format import *
from .indent import *
from .index import *
from .join import *
from .jsondecode import *
from .jsonencode import *
from .keys import *
from .length import *
from .log import *
from .lookup import *
from .lower import *
from .map import *
from .matchkeys import *
from .max import *
from .md5 import *
from .merge import *
from .min import *
from .parseint import *
from .pathexpand import *
from .pow import *
from .provider import *
from .range import *
from .replace import *
from .reverse import *
from .rsadecrypt import *
from .sha1 import *
from .sha256 import *
from .sha512 import *
from .signum import *
from .slice import *
from .sort import *
from .split import *
from .startswith import *
from .strrev import *
from .substr import *
from .sum import *
from .timeadd import *
from .timecmp import *
from .timestamp import *
from .title import *
from .tobool import *
from .tolist import *
from .tonumber import *
from .toset import *
from .tostring import *
from .transpose import *
from .trim import *
from .trimprefix import *
from .trimspace import *
from .trimsuffix import *
from .upper import *
from .urlencode import *
from .uuid import *
from .values import *
from .zipmap import *
_utilities.register(
    resource_modules="""
[]
""",
    resource_packages="""
[
 {
  "pkg": "std",
  "token": "pulumi:providers:std",
  "fqn": "pulumi_std",
  "class": "Provider"
 }
]
"""
)



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = ['ProviderArgs', 'Provider']

@pulumi.input_type
class ProviderArgs:
    def __init__(__self__):
        """
        The set of arguments for constructing a Provider resource.
        """
        pass


class Provider(pulumi.ProviderResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 __props__=None):
        """
        Create a Std resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: Optional[ProviderArgs] = None,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Create a Std resource with the given unique name, props, and options.
        :param str resource_name: The name of the resource.
        :param ProviderArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ProviderArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ProviderArgs.__new__(ProviderArgs)

        super(Provider, __self__).__init__(
            'std',
            resource_name,
            __props__,
            opts)




# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'MapResult',
    'AwaitableMapResult',
    'map',
    'map_output',
]

@pulumi.output_type
class MapResult:
    def __init__(__self__, result=None):
        if result and not isinstance(result, dict):
            raise TypeError("Expected argument 'result' to be a dict")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> Mapping[str, Any]:
        return pulumi.get(self, "result")


class AwaitableMapResult(MapResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return MapResult(
            result=self.result)


def map(args: Optional[Sequence[Any]] = None,
        opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableMapResult:
    """
    Returns a map consisting of the key/value pairs specified as arguments.
    """
    __args__ = dict()
    __args__['args'] = args
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:map', __args__, opts=opts, typ=MapResult).value

    return AwaitableMapResult(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(map)
def map_output(args: Optional[pulumi.Input[Sequence[Any]]] = None,
               opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[MapResult]:
    """
    Returns a map consisting of the key/value pairs specified as arguments.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'IndentResult',
    'AwaitableIndentResult',
    'indent',
    'indent_output',
]

@pulumi.output_type
class IndentResult:
    def __init__(__self__, result=None):
        if result and not isinstance(result, str):
            raise TypeError("Expected argument 'result' to be a str")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> str:
        return pulumi.get(self, "result")


class AwaitableIndentResult(IndentResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return IndentResult(
            result=self.result)


def indent(input: Optional[str] = None,
           spaces: Optional[int] = None,
           opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableIndentResult:
    """
    Adds a given number of spaces after each newline character in the given string.
    """
    __args__ = dict()
    __args__['input'] = input
    __args__['spaces'] = spaces
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:indent', __args__, opts=opts, typ=IndentResult).value

    return AwaitableIndentResult(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(indent)
def indent_output(input: Optional[pulumi.Input[str]] = None,
                  spaces: Optional[pulumi.Input[int]] = None,
                  opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[IndentResult]:
    """
    Adds a given number of spaces after each newline character in the given string.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'RangeResult',
    'AwaitableRangeResult',
    'range',
    'range_output',
]

@pulumi.output_type
class RangeResult:
    def __init__(__self__, result=None):
        if result and not isinstance(result, list):
            raise TypeError("Expected argument 'result' to be a list")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> Sequence[float]:
        return pulumi.get(self, "result")


class AwaitableRangeResult(RangeResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return RangeResult(
            result=self.result)


def range(limit: Optional[float] = None,
          start: Optional[float] = None,
          step: Optional[float] = None,
          opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableRangeResult:
    """
    Generates a list of numbers using a start value, a limit value, and a step value.
    Start and step may be omitted, in which case start defaults to zero and step defaults to either one or negative one
    depending on whether limit is greater than or less than start.
    """
    __args__ = dict()
    __args__['limit'] = limit
    __args__['start'] = start
    __args__['step'] = step
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:range', __args__, opts=opts, typ=RangeResult).value

    return AwaitableRangeResult(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(range)
def range_output(limit: Optional[pulumi.Input[float]] = None,
                 start: Optional[pulumi.Input[Optional[float]]] = None,
                 step: Optional[pulumi.Input[Optional[float]]] = None,
                 opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[RangeResult]:
    """
    Generates a list of numbers using a start value, a limit value, and a step value.
    Start and step may be omitted, in which case start defaults to zero and step defaults to either one or negative one
    depending on whether limit is greater than or less than start.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'TimecmpResult',
    'AwaitableTimecmpResult',
    'timecmp',
    'timecmp_output',
]

@pulumi.output_type
class TimecmpResult:
    def __init__(__self__, result=None):
        if result and not isinstance(result, int):
            raise TypeError("Expected argument 'result' to be a int")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> int:
        return pulumi.get(self, "result")


class AwaitableTimecmpResult(TimecmpResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return TimecmpResult(
            result=self.result)


def timecmp(timestampa: Optional[str] = None,
            timestampb: Optional[str] = None,
            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableTimecmpResult:
    """
    Compares two timestamps and returns a number that represents the ordering
    	of the instants those timestamps represent.
    	Timestamps are represented as strings using RFC 3339 "Date and time format" syntax.
    	Both timestamps must be strings adhering this syntax, i.e. "2017-11-22T00:00:00Z".
    	If 'timestamp_a' is before 'timestamp_b', -1 is returned.
    	If 'timestamp_a' is equal to 'timestamp_b', 0 is returned.
    	If 'timestamp_a' is after 'timestamp_b', 1 is returned.
    """
    __args__ = dict()
    __args__['timestampa'] = timestampa
    __args__['timestampb'] = timestampb
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:timecmp', __args__, opts=opts, typ=TimecmpResult).value

    return AwaitableTimecmpResult(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(timecmp)
def timecmp_output(timestampa: Optional[pulumi.Input[str]] = None,
                   timestampb: Optional[pulumi.Input[str]] = None,
                   opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[TimecmpResult]:
    """
    Compares two timestamps and returns a number that represents the ordering
    	of the instants those timestamps represent.
    	Timestamps are represented as strings using RFC 3339 "Date and time format" syntax.
    	Both timestamps must be strings adhering this syntax, i.e. "2017-11-22T00:00:00Z".
    	If 'timestamp_a' is before 'timestamp_b', -1 is returned.
    	If 'timestamp_a' is equal to 'timestamp_b', 0 is returned.
    	If 'timestamp_a' is after 'timestamp_b', 1 is returned.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'AbspathResult',
    'AwaitableAbspathResult',
    'abspath',
    'abspath_output',
]

@pulumi.output_type
class AbspathResult:
    def __init__(__self__, result=None):
        if result and not isinstance(result, str):
            raise TypeError("Expected argument 'result' to be a str")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> str:
        return pulumi.get(self, "result")


class AwaitableAbspathResult(AbspathResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return AbspathResult(
            result=self.result)


def abspath(input: Optional[str] = None,
            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableAbspathResult:
    """
    Returns an absolute representation of the specified path.
    If the path is not absolute it will be joined with the current working directory to turn it into an absolute path.
    """
    __args__ = dict()
    __args__['input'] = input
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:abspath', __args__, opts=opts, typ=AbspathResult).value

    return AwaitableAbspathResult(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(abspath)
def abspath_output(input: Optional[pulumi.Input[str]] = None,
                   opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[AbspathResult]:
    """
    Returns an absolute representation of the specified path.
    If the path is not absolute it will be joined with the current working directory to turn it into an absolute path.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'TimeaddResult',
    'AwaitableTimeaddResult',
    'timeadd',
    'timeadd_output',
]

@pulumi.output_type
class TimeaddResult:
    def __init__(__self__, result=None):
        if result and not isinstance(result, str):
            raise TypeError("Expected argument 'result' to be a str")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> str:
        return pulumi.get(self, "result")


class AwaitableTimeaddResult(TimeaddResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return TimeaddResult(
            result=self.result)


def timeadd(duration: Optional[str] = None,
            timestamp: Optional[str] = None,
            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableTimeaddResult:
    """
    Adds a duration to a timestamp, returning a new timestamp.
    	Timestamps are represented as strings using RFC 3339 "Date and time format" syntax.
    	'timestamp' must be a string adhering this syntax, i.e. "2017-11-22T00:00:00Z".
    	'duration' is a string representation of a time difference, comprised of sequences of
    	numbers and unit pairs, i.e. "3.5h" or "2h15m".
    	Accepted units are "ns", "us" or "µs", "ms", "s", "m", and "h". The first number may be negative
    	to provide a negative duration, i.e. "-2h15m".
    """
    __args__ = dict()
    __args__['duration'] = duration
    __args__['timestamp'] = timestamp
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:timeadd', __args__, opts=opts, typ=TimeaddResult).value

    return AwaitableTimeaddResult(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(timeadd)
def timeadd_output(duration: Optional[pulumi.Input[str]] = None,
                   timestamp: Optional[pulumi.Input[str]] = None,
                   opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[TimeaddResult]:
    """
    Adds a duration to a timestamp, returning a new timestamp.
    	Timestamps are represented as strings using RFC 3339 "Date and time format" syntax.
    	'timestamp' must be a string adhering this syntax, i.e. "2017-11-22T00:00:00Z".
    	'duration' is a string representation of a time difference, comprised of sequences of
    	numbers and unit pairs, i.e. "3.5h" or "2h15m".
    	Accepted units are "ns", "us" or "µs", "ms", "s", "m", and "h". The first number may be negative
    	to provide a negative duration, i.e. "-2h15m".
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'CsvdecodeResult',
    'AwaitableCsvdecodeResult',
    'csvdecode',
    'csvdecode_output',
]

@pulumi.output_type
class CsvdecodeResult:
    def __init__(__self__, result=None):
        if result and not isinstance(result, list):
            raise TypeError("Expected argument 'result' to be a list")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> Sequence[Mapping[str, str]]:
        return pulumi.get(self, "result")


class AwaitableCsvdecodeResult(CsvdecodeResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return CsvdecodeResult(
            result=self.result)


def csvdecode(input: Optional[str] = None,
              opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableCsvdecodeResult:
    """
    Decodes a string containing CSV-formatted data and produces a list of maps representing that data.
    	The first line of the CSV data is interpreted as a "header" row: the values given
    	are used as the keys in the resulting maps.
    	Each subsequent line becomes a single map in the resulting list,
    	matching the keys from the header row with the given values by index.
    	All lines in the file must contain the same number of fields,
    	or this function will produce an error.
    	Follows the format defined in RFC 4180.
    """
    __args__ = dict()
    __args__['input'] = input
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:csvdecode', __args__, opts=opts, typ=CsvdecodeResult).value

    return AwaitableCsvdecodeResult(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(csvdecode)
def csvdecode_output(input: Optional[pulumi.Input[str]] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[CsvdecodeResult]:
    """
    Decodes a string containing CSV-formatted data and produces a list of maps representing that data.
    	The first line of the CSV data is interpreted as a "header" row: the values given
    	are used as the keys in the resulting maps.
    	Each subsequent line becomes a single map in the resulting list,
    	matching the keys from the header row with the given values by index.
    	All lines in the file must contain the same number of fields,
    	or this function will produce an error.
    	Follows the format defined in RFC 4180.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***


import asyncio
import functools
import importlib.metadata
import importlib.util
import inspect
import json
import os
import sys
import typing
import warnings

import pulumi
import pulumi.runtime
from pulumi.runtime.sync_await import _sync_await

from semver import VersionInfo as SemverVersion
from parver import Version as PEP440Version

C = typing.TypeVar("C", bound=typing.Callable)


def get_env(*args):
    for v in args:
        value = os.getenv(v)
        if value is not None:
            return value
    return None


def get_env_bool(*args):
    str = get_env(*args)
    if str is not None:
        # NOTE: these values are taken from https://golang.org/src/strconv/atob.go?s=351:391#L1, which is what
        # Terraform uses internally when parsing boolean values.
        if str in ["1", "t", "T", "true", "TRUE", "True"]:
            return True
        if str in ["0", "f", "F", "false", "FALSE", "False"]:
            return False
    return None


def get_env_int(*args):
    str = get_env(*args)
    if str is not None:
        try:
            return int(str)
        except:
            return None
    return None


def get_env_float(*args):
    str = get_env(*args)
    if str is not None:
        try:
            return float(str)
        except:
            return None
    return None


def _get_semver_version():
    # __name__ is set to the fully-qualified name of the current module, In our case, it will be
    # <some module>._utilities. <some module> is the module we want to query the version for.
    root_package, *rest = __name__.split('.')

    # pkg_resources uses setuptools to inspect the set of installed packages. We use it here to ask
    # for the currently installed version of the root package (i.e. us) and get its version.

    # Unfortunately, PEP440 and semver differ slightly in incompatible ways. The Pulumi engine expects
    # to receive a valid semver string when receiving requests from the language host, so it's our
    # responsibility as the library to convert our own PEP440 version into a valid semver string.

    pep440_version_string = importlib.metadata.version(root_package)
    pep440_version = PEP440Version.parse(pep440_version_string)
    (major, minor, patch) = pep440_version.release
    prerelease = None
    if pep440_version.pre_tag == 'a':
        prerelease = f"alpha.{pep440_version.pre}"
    elif pep440_version.pre_tag == 'b':
        prerelease = f"beta.{pep440_version.pre}"
    elif pep440_version.pre_tag == 'rc':
        prerelease = f"rc.{pep440_version.pre}"
    elif pep440_version.dev is not None:
        prerelease = f"dev.{pep440_version.dev}"

    # The only significant difference between PEP440 and semver as it pertains to us is that PEP440 has explicit support
    # for dev builds, while semver encodes them as "prerelease" versions. In order to bridge between the two, we convert
    # our dev build version into a prerelease tag. This matches what all of our other packages do when constructing
    # their own semver string.
    return SemverVersion(major=major, minor=minor, patch=patch, prerelease=prerelease)


# Determine the version once and cache the value, which measurably improves program performance.
_version = _get_semver_version()
_version_str = str(_version)


def get_version():
    return _version_str

def get_resource_opts_defaults() -> pulumi.ResourceOptions:
    return pulumi.ResourceOptions(
        version=get_version(),
        plugin_download_url=get_plugin_download_url(),
    )

def get_invoke_opts_defaults() -> pulumi.InvokeOptions:
    return pulumi.InvokeOptions(
        version=get_version(),
        plugin_download_url=get_plugin_download_url(),
    )

def get_resource_args_opts(resource_args_type, resource_options_type, *args, **kwargs):
    """
    Return the resource args and options given the *args and **kwargs of a resource's
    __init__ method.
    """

    resource_args, opts = None, None

    # If the first item is the resource args type, save it and remove it from the args list.
    if args and isinstance(args[0], resource_args_type):
        resource_args, args = args[0], args[1:]

    # Now look at the first item in the args list again.
    # If the first item is the resource options class, save it.
    if args and isinstance(args[0], resource_options_type):
        opts = args[0]

    # If resource_args is None, see if "args" is in kwargs, and, if so, if it's typed as the
    # the resource args type.
    if resource_args is None:
        a = kwargs.get("args")
        if isinstance(a, resource_args_type):
            resource_args = a

    # If opts is None, look it up in kwargs.
    if opts is None:
        opts = kwargs.get("opts")

    return resource_args, opts


# Temporary: just use pulumi._utils.lazy_import once everyone upgrades.
def lazy_import(fullname):

    import pulumi._utils as u
    f = getattr(u, 'lazy_import', None)
    if f is None:
        f = _lazy_import_temp

    return f(fullname)


# Copied from pulumi._utils.lazy_import, see comments there.
def _lazy_import_temp(fullname):
    m = sys.modules.get(fullname, None)
    if m is not None:
        return m

    spec = importlib.util.find_spec(fullname)

    m = sys.modules.get(fullname, None)
    if m is not None:
        return m

    loader = importlib.util.LazyLoader(spec.loader)
    spec.loader = loader
    module = importlib.util.module_from_spec(spec)

    m = sys.modules.get(fullname, None)
    if m is not None:
        return m

    sys.modules[fullname] = module
    loader.exec_module(module)
    return module


class Package(pulumi.runtime.ResourcePackage):
    def __init__(self, pkg_info):
        super().__init__()
        self.pkg_info = pkg_info

    def version(self):
        return _version

    def construct_provider(self, name: str, typ: str, urn: str) -> pulumi.ProviderResource:
        if typ != self.pkg_info['token']:
            raise Exception(f"unknown provider type {typ}")
        Provider = getattr(lazy_import(self.pkg_info['fqn']), self.pkg_info['class'])
        return Provider(name, pulumi.ResourceOptions(urn=urn))


class Module(pulumi.runtime.ResourceModule):
    def __init__(self, mod_info):
        super().__init__()
        self.mod_info = mod_info

    def version(self):
        return _version

    def construct(self, name: str, typ: str, urn: str) -> pulumi.Resource:
        class_name = self.mod_info['classes'].get(typ, None)

        if class_name is None:
            raise Exception(f"unknown resource type {typ}")

        TheClass = getattr(lazy_import(self.mod_info['fqn']), class_name)
        return TheClass(name, pulumi.ResourceOptions(urn=urn))


def register(resource_modules, resource_packages):
    resource_modules = json.loads(resource_modules)
    resource_packages = json.loads(resource_packages)

    for pkg_info in resource_packages:
        pulumi.runtime.register_resource_package(pkg_info['pkg'], Package(pkg_info))

    for mod_info in resource_modules:
        pulumi.runtime.register_resource_module(
            mod_info['pkg'],
            mod_info['mod'],
            Module(mod_info))


_F = typing.TypeVar('_F', bound=typing.Callable[..., typing.Any])


def lift_output_func(func: typing.Any) -> typing.Callable[[_F], _F]:
    """Decorator internally used on {fn}_output lifted function versions
    to implement them automatically from the un-lifted function."""

    func_sig = inspect.signature(func)

    def lifted_func(*args, opts=None, **kwargs):
        bound_args = func_sig.bind(*args, **kwargs)
        # Convert tuple to list, see pulumi/pulumi#8172
        args_list = list(bound_args.args)
        return pulumi.Output.from_input({
            'args': args_list,
            'kwargs': bound_args.kwargs
        }).apply(lambda resolved_args: func(*resolved_args['args'],
                                            opts=opts,
                                            **resolved_args['kwargs']))

    return (lambda _: lifted_func)


def call_plain(
    tok: str,
    props: pulumi.Inputs,
    res: typing.Optional[pulumi.Resource] = None,
    typ: typing.Optional[type] = None,
) -> typing.Any:
    """
    Wraps pulumi.runtime.plain to force the output and return it plainly.
    """

    output = pulumi.runtime.call(tok, props, res, typ)

    # Ingoring deps silently. They are typically non-empty, r.f() calls include r as a dependency.
    result, known, secret, _ = _sync_await(asyncio.ensure_future(_await_output(output)))

    problem = None
    if not known:
        problem = ' an unknown value'
    elif secret:
        problem = ' a secret value'

    if problem:
        raise AssertionError(
            f"Plain resource method '{tok}' incorrectly returned {problem}. "
            + "This is an error in the provider, please report this to the provider developer."
        )

    return result


async def _await_output(o: pulumi.Output[typing.Any]) -> typing.Tuple[object, bool, bool, set]:
    return (
        await o._future,
        await o._is_known,
        await o._is_secret,
        await o._resources,
    )


# This is included to provide an upgrade path for users who are using a version
# of the Pulumi SDK (<3.121.0) that does not include the `deprecated` decorator.
def deprecated(message: str) -> typing.Callable[[C], C]:
    """
    Decorator to indicate a function is deprecated.

    As well as inserting appropriate statements to indicate that the function is
    deprecated, this decorator also tags the function with a special attribute
    so that Pulumi code can detect that it is deprecated and react appropriately
    in certain situations.

    message is the deprecation message that should be printed if the function is called.
    """

    def decorator(fn: C) -> C:
        if not callable(fn):
            raise TypeError("Expected fn to be callable")

        @functools.wraps(fn)
        def deprecated_fn(*args, **kwargs):
            warnings.warn(message)
            pulumi.warn(f"{fn.__name__} is deprecated: {message}")

            return fn(*args, **kwargs)

        deprecated_fn.__dict__["_pulumi_deprecated_callable"] = fn
        return typing.cast(C, deprecated_fn)

    return decorator

def get_plugin_download_url():
	return None



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'TransposeResult',
    'AwaitableTransposeResult',
    'transpose',
    'transpose_output',
]

@pulumi.output_type
class TransposeResult:
    def __init__(__self__, result=None):
        if result and not isinstance(result, dict):
            raise TypeError("Expected argument 'result' to be a dict")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> Mapping[str, Sequence[str]]:
        return pulumi.get(self, "result")


class AwaitableTransposeResult(TransposeResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return TransposeResult(
            result=self.result)


def transpose(input: Optional[Mapping[str, Sequence[str]]] = None,
              opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableTransposeResult:
    """
    Takes a map of lists of strings and swaps the keys and values to return a new map of lists of strings.
    """
    __args__ = dict()
    __args__['input'] = input
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:transpose', __args__, opts=opts, typ=TransposeResult).value

    return AwaitableTransposeResult(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(transpose)
def transpose_output(input: Optional[pulumi.Input[Mapping[str, Sequence[str]]]] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[TransposeResult]:
    """
    Takes a map of lists of strings and swaps the keys and values to return a new map of lists of strings.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'Filemd5Result',
    'AwaitableFilemd5Result',
    'filemd5',
    'filemd5_output',
]

@pulumi.output_type
class Filemd5Result:
    def __init__(__self__, result=None):
        if result and not isinstance(result, str):
            raise TypeError("Expected argument 'result' to be a str")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> str:
        return pulumi.get(self, "result")


class AwaitableFilemd5Result(Filemd5Result):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return Filemd5Result(
            result=self.result)


def filemd5(input: Optional[str] = None,
            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableFilemd5Result:
    """
    Reads the contents of a file into a string and returns the MD5 hash of it.
    """
    __args__ = dict()
    __args__['input'] = input
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:filemd5', __args__, opts=opts, typ=Filemd5Result).value

    return AwaitableFilemd5Result(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(filemd5)
def filemd5_output(input: Optional[pulumi.Input[str]] = None,
                   opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[Filemd5Result]:
    """
    Reads the contents of a file into a string and returns the MD5 hash of it.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'MatchkeysResult',
    'AwaitableMatchkeysResult',
    'matchkeys',
    'matchkeys_output',
]

@pulumi.output_type
class MatchkeysResult:
    def __init__(__self__, result=None):
        if result and not isinstance(result, list):
            raise TypeError("Expected argument 'result' to be a list")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> Sequence[str]:
        return pulumi.get(self, "result")


class AwaitableMatchkeysResult(MatchkeysResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return MatchkeysResult(
            result=self.result)


def matchkeys(search_list: Optional[Sequence[str]] = None,
              values: Optional[Sequence[str]] = None,
              opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableMatchkeysResult:
    """
    For two lists values and keys of equal length,
    returns all elements from values where the corresponding element from keys exists in the searchset list.
    """
    __args__ = dict()
    __args__['searchList'] = search_list
    __args__['values'] = values
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:matchkeys', __args__, opts=opts, typ=MatchkeysResult).value

    return AwaitableMatchkeysResult(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(matchkeys)
def matchkeys_output(search_list: Optional[pulumi.Input[Sequence[str]]] = None,
                     values: Optional[pulumi.Input[Sequence[str]]] = None,
                     opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[MatchkeysResult]:
    """
    For two lists values and keys of equal length,
    returns all elements from values where the corresponding element from keys exists in the searchset list.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'ReplaceResult',
    'AwaitableReplaceResult',
    'replace',
    'replace_output',
]

@pulumi.output_type
class ReplaceResult:
    def __init__(__self__, result=None):
        if result and not isinstance(result, str):
            raise TypeError("Expected argument 'result' to be a str")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> str:
        return pulumi.get(self, "result")


class AwaitableReplaceResult(ReplaceResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return ReplaceResult(
            result=self.result)


def replace(replace: Optional[str] = None,
            search: Optional[str] = None,
            text: Optional[str] = None,
            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableReplaceResult:
    """
    Does a search and replace on the given string.
    All instances of search are replaced with the value of replace.
    If search is wrapped in forward slashes, it is treated as a regular expression.
    If using a regular expression, replace can reference subcaptures in the regular expression by
    using $n where n is the index or name of the subcapture. If using a regular expression,
    the syntax conforms to the re2 regular expression syntax.
    """
    __args__ = dict()
    __args__['replace'] = replace
    __args__['search'] = search
    __args__['text'] = text
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:replace', __args__, opts=opts, typ=ReplaceResult).value

    return AwaitableReplaceResult(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(replace)
def replace_output(replace: Optional[pulumi.Input[str]] = None,
                   search: Optional[pulumi.Input[str]] = None,
                   text: Optional[pulumi.Input[str]] = None,
                   opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[ReplaceResult]:
    """
    Does a search and replace on the given string.
    All instances of search are replaced with the value of replace.
    If search is wrapped in forward slashes, it is treated as a regular expression.
    If using a regular expression, replace can reference subcaptures in the regular expression by
    using $n where n is the index or name of the subcapture. If using a regular expression,
    the syntax conforms to the re2 regular expression syntax.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'CidrhostResult',
    'AwaitableCidrhostResult',
    'cidrhost',
    'cidrhost_output',
]

@pulumi.output_type
class CidrhostResult:
    def __init__(__self__, result=None):
        if result and not isinstance(result, str):
            raise TypeError("Expected argument 'result' to be a str")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> str:
        return pulumi.get(self, "result")


class AwaitableCidrhostResult(CidrhostResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return CidrhostResult(
            result=self.result)


def cidrhost(host: Optional[int] = None,
             input: Optional[str] = None,
             opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableCidrhostResult:
    """
    Takes an IP address range in CIDR notation as input
    and creates an IP address with the given host number.
    If given host number is negative, the count starts from the end of the range.
    For example, cidrhost("10.0.0.0/8", 2) returns 10.0.0.2 and cidrhost("10.0.0.0/8", -2) returns 10.255.255.254.
    """
    __args__ = dict()
    __args__['host'] = host
    __args__['input'] = input
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:cidrhost', __args__, opts=opts, typ=CidrhostResult).value

    return AwaitableCidrhostResult(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(cidrhost)
def cidrhost_output(host: Optional[pulumi.Input[int]] = None,
                    input: Optional[pulumi.Input[str]] = None,
                    opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[CidrhostResult]:
    """
    Takes an IP address range in CIDR notation as input
    and creates an IP address with the given host number.
    If given host number is negative, the count starts from the end of the range.
    For example, cidrhost("10.0.0.0/8", 2) returns 10.0.0.2 and cidrhost("10.0.0.0/8", -2) returns 10.255.255.254.
    """
    ...



# coding=utf-8
# *** WARNING: this file was generated by pulumi-language-python. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
from . import _utilities

__all__ = [
    'SliceResult',
    'AwaitableSliceResult',
    'slice',
    'slice_output',
]

@pulumi.output_type
class SliceResult:
    def __init__(__self__, result=None):
        if result and not isinstance(result, list):
            raise TypeError("Expected argument 'result' to be a list")
        pulumi.set(__self__, "result", result)

    @property
    @pulumi.getter
    def result(self) -> Sequence[Any]:
        return pulumi.get(self, "result")


class AwaitableSliceResult(SliceResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return SliceResult(
            result=self.result)


def slice(from_: Optional[int] = None,
          list: Optional[Sequence[Any]] = None,
          to: Optional[int] = None,
          opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableSliceResult:
    """
    Returns the portion of list between from (inclusive) and to (exclusive).
    """
    __args__ = dict()
    __args__['from'] = from_
    __args__['list'] = list
    __args__['to'] = to
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('std:index:slice', __args__, opts=opts, typ=SliceResult).value

    return AwaitableSliceResult(
        result=pulumi.get(__ret__, 'result'))


@_utilities.lift_output_func(slice)
def slice_output(from_: Optional[pulumi.Input[Optional[int]]] = None,
                 list: Optional[pulumi.Input[Sequence[Any]]] = None,
                 to: Optional[pulumi.Input[Optional[int]]] = None,
                 opts: Optional[pulumi.InvokeOptions] = None) -> pulumi.Output[SliceResult]:
    """
    Returns the portion of list between from (inclusive) and to (exclusive).
    """
    ...
